# @summary
#   profile for puppetmaster
#
# @param public_mirror
#   where to sync the code repository to
#
class tails_profile_puppet::master (
  String                                   $public_mirror  = 'git@gitlab-ssh.tails.boum.org:tails/puppet-code',
) {
  include rbac

  file { [
      '/opt',
      '/opt/puppetlabs',
      '/opt/puppetlabs/server',
      '/opt/puppetlabs/server/apps',
      '/opt/puppetlabs/server/apps/puppetserver',
      '/etc/puppet/conf.d',
    ]:
      ensure => directory,
  }

# dirty hack to prevent the puppetmodule from resetting vardir ownership
# and restarting puppet master all the time
  File <| title == '/var/lib/puppet' |> {
    owner => undef,
    group => undef,
  }
  File <| title == '/var/cache/puppet' |> {
    owner => undef,
    group => undef,
  }

  class { 'puppet':
    manage_packages          => true,
    agent                    => true,
    runmode                  => 'systemd.timer',
    server                   => true,
    server_reports           => 'store,puppetdb',
    server_foreman           => false,
    server_external_nodes    => '',
    server_git_repo          => true,
    server_git_repo_path     => '/var/lib/puppet/puppet-code.git',
    server_post_hook_content => 'tails_profile_puppet/master/deploy_environments.py',
    server_storeconfigs      => true,
    additional_settings      => {
      fact_value_length_soft_limit => 65535,
      include_legacy_facts         => true,
    },
  } -> Class['tails_profile_puppet::master::g10k']

  include tails_profile_puppet::master::g10k

  class { 'puppet::server::puppetdb':
    server => $facts['networking']['fqdn'],
  }

# ensure all commits to our puppet repo are signed

  file { '/var/lib/puppet/puppet-code.git/hooks/update':
    source  => 'puppet:///modules/tails_profile_puppet/master/check-commit-signature',
    owner   => 'puppet',
    group   => 'puppet',
    mode    => '0755',
    require => Class['puppet'],
  }

# ensure we have a remote for our public mirror

  exec { "/usr/bin/git -C /var/lib/puppet/puppet-code.git remote add mirror ${public_mirror}":
    user   => puppet,
    group  => puppet,
    unless => '/usr/bin/git -C /var/lib/puppet/puppet-code.git remote |grep -qs -x mirror';
  }

# and set up a hook to sync to the public mirror

  file { '/var/lib/puppet/puppet-code.git/hooks/post-update':
    source  => 'puppet:///modules/tails_profile_puppet/master/gitlab-mirror',
    owner   => 'puppet',
    group   => 'puppet',
    mode    => '0755',
    require => Class['puppet'],
  }
# ensure we have backups for the puppetdb

  include backupninja
  backupninja::pgsql { 'backup-puppetdb':
    ensure => present,
  }

# make sure we can encrypt sensitive data in hiera

  include tails_profile_puppet::eyaml

# ensure sysadmins can access the git repo

  rbac::ssh { 'sysadmins-to-puppet':
    user => 'puppet',
    role => 'sysadmins',
  }

# ensure puppet knows the sysadmin PGP keys

  include yapgp
  $rbac::roles['sysadmins'].each | String $username | {
    $fp = $rbac::users[$username]['pgpkey']

    pgp_key { "puppet-${fp}":
      ensure => present,
      fp     => $fp,
      user   => puppet,
      trust  => 5,
    }
  }

# Clean up reports

  exec { '/usr/bin/find /var/lib/puppetserver/reports -type f -ctime +2 -exec /bin/rm {} \;': }

# Firewall

  tirewall::accept_trusted_subnets { 'Puppet Master':
    dport => 8140,
  }

# Log rotation

  ensure_packages(['logrotate'])

  file { '/etc/logrotate.d/puppet':
    ensure  => file,
    owner   => root,
    group   => root,
    mode    => '0644',
    source  => 'puppet:///modules/tails_profile_puppet/master/logrotate.conf',
    require => Package['logrotate'],
  }

# rexml gem

  # XXX The rexml gem was removed from the Puppet Server Debian package in
  # Trixie, but we need it for the `cirrax-libvirt` module. Below, we install
  # it from the libruby3.3 package and symlink it into where JRuby will find
  # it.
  #
  # This is not ideal because the symlink will likely break during the next
  # release upgrade, when the gems is changed to a newer version.
  #
  # Also, this puppet module will be ditched soon in favor of TPA's, so this
  # code will need to be ported there.
  #
  # We should review this, at least in the light of the merge of virtualization
  # schemes, but the current plan is that will only be tackled some years from
  # now (2029).
  package { 'libruby3.3': ensure => installed }

  file { '/usr/lib/puppetserver/vendored-jruby-gems/gems/rexml-3.3.9':
    ensure  => link,
    target  => '/usr/lib/ruby/gems/3.3.0/gems/rexml-3.3.9',
    notify  => Service['puppetserver'],
    require => Package[
      'libruby3.3',
      'puppetserver',
    ],
  }

  file { '/usr/lib/puppetserver/vendored-jruby-gems/specifications/rexml-3.3.9.gemspec':
    ensure  => link,
    target  => '/usr/lib/ruby/gems/3.3.0/specifications/rexml-3.3.9.gemspec',
    notify  => Service['puppetserver'],
    require => Package[
      'libruby3.3',
      'puppetserver',
    ],
  }
}
