#!/usr/bin/python3

import logging
import os
import re
import shutil
import sys
import subprocess
import yaml


G10K_CONFIG_FILE = "/etc/g10k.yaml"
PUPPET_CODEDIR = "/etc/puppet/code"

logging.basicConfig(level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s")
logger = logging.getLogger(__name__)


def delete_env(branchname, envdir):
    try:
        logger.info(f"Deleting environment {branchname} at {envdir}...")
        shutil.rmtree(envdir)
    except FileNotFoundError:
        logger.error(f"Environment directory {envdir} not found")
    except OSError as e:
        logger.error(f"Error while deleting environment: {e}")


def ensure_env(branchname, envdir, cachedir, g10k_config_file):
    # create or update new environment
    logger.info(f"Creating or updating environment {branchname}...")
    subprocess.run(
        ["/usr/bin/g10k", "-config", g10k_config_file, "-branch", branchname],
        check=True,
    )
    envname = re.sub(r"\W", "_", branchname)
    subprocess.run(
        ["/usr/bin/puppet", "generate", "--codedir", PUPPET_CODEDIR, "types", "--environment", envname],
        check=False,
    )


def process_line(line, basedir, cachedir, g10k_config_file=G10K_CONFIG_FILE):
    _, newrev, ref = line.split()
    branchname = re.sub(r"^refs/heads/", "", ref)

    envdir = os.path.join(basedir, re.sub(r"\W", "_", branchname))

    if re.match("^0+$", newrev):
        delete_env(branchname, envdir)
    else:
        ensure_env(branchname, envdir, cachedir, g10k_config_file=g10k_config_file)


def post_receive(basedir, cachedir):
    for line in sys.stdin.readlines():
        line = line.strip()
        process_line(line, basedir, cachedir)


def get_config():
    with open("/etc/g10k.yaml", "r") as file:
        data = yaml.safe_load(file)
    return data


def main():
    config = get_config()
    basedir = config["sources"]["puppet"]["basedir"]
    cachedir = config[":cachedir"]

    if not os.path.isdir(basedir):
        logger.error(f"{basedir} does not exist, cannot create environment directories.")
        sys.exit(1)

    post_receive(basedir, cachedir)


if __name__ == "__main__":
    main()
