#!/usr/bin/python3

import filecmp
import os
import pytest
import re
import subprocess
import tempfile

import deploy_environments


@pytest.fixture
def basedir(tmp_path):
    """
    The base directory where g10k creates environments.
    """
    return os.path.join(tmp_path, "environments")


@pytest.fixture(scope="session")
def cachedir(tmp_path_factory):
    """
    A cache directory for g10k.
    """
    cachedir = os.getenv(
        "G10K_CACHEDIR",
        tmp_path_factory.mktemp("cache"),
    )
    os.makedirs(cachedir, exist_ok=True)
    return cachedir


@pytest.fixture(params=[False, True])
def branchname(repodir, request):
    """
    The name of the branch to be deployed by g10k.

    When run in GitLab CI, get the branch name from the CI_COMMIT_REF_NAME
    environment variable. Else, get from Git.

    We parameterized the branchname to make sure we always test branches
    containing non-word characters.
    """
    branchname = os.getenv(
        "CI_COMMIT_REF_NAME",
        subprocess.check_output(
            ["git", "rev-parse", "--abbrev-ref", "HEAD"],
            text=True,
        ).strip(),
    )
    subprocess.run(
        f"git clone --branch={branchname} . {repodir}",
        shell=True,
        check=True,
    )

    if request.param:
        branchname = branchname + "-man/gled"
        subprocess.run(
            f"git -C {repodir} checkout -b {branchname}",
            shell=True,
            check=True,
        )

    return branchname


@pytest.fixture
def envdir(tmp_path, basedir, branchname):
    """
    The environment directory.
    """
    # g10k replaces non-word characters in the branchname to come up with the
    # environment name.
    envname = re.sub(r"\W", "_", branchname)
    return os.path.join(basedir, envname)


@pytest.fixture
def repodir(tmp_path):
    """
    The directory for a clone of the repository that we can play with without
    interfering in the main repository.
    """
    repodir = os.path.join(tmp_path, "repo")
    return repodir


@pytest.fixture
def configfile(tmp_path, cachedir, repodir, basedir):
    """
    The path for a valid g10k config file to be used on tests.
    """
    filename = os.path.join(tmp_path, "g10k.yaml")
    with open(filename, "w") as f:
        f.writelines(
            f"""---
:cachedir: {cachedir}
sources:
  puppet:
    remote: {repodir}
    basedir: {basedir}
    prefix: false
    force_forge_versions: true
    warn_if_branch_is_missing: true"""
        )
    return filename


@pytest.fixture
def modules(repodir):
    """
    The list of modules declared in the Puppetfile in repodir.
    """
    puppetfile_path = os.path.join(repodir, "Puppetfile")
    modules = []
    with open(puppetfile_path, "r") as f:
        for line in f:
            match = re.match(r'\s*mod\s+["\']([^"\']+)["\']', line)
            if match:
                modname = re.sub(r"-", "/", match.group(1)).split("/")[1]
                modules.append(modname)
    return modules


def test_process_line_installs_modules(branchname, basedir, cachedir, envdir, repodir, configfile, modules):
    "Updating a branch installs modules in the environment."
    line = f"abc123 def456 refs/heads/{branchname}"
    deploy_environments.process_line(
        line,
        basedir,
        cachedir,
        g10k_config_file=configfile,
    )

    assert len(modules) > 0

    # make sure all modules in the Puppetfile were installed
    for module in modules:
        module_path = os.path.join(envdir, "modules", module)
        assert os.path.isdir(module_path) is True


def test_process_line_creates_environment(branchname, basedir, cachedir, envdir, repodir, configfile, tmp_path):
    "Updating a branch creates an environment."
    line = f"abc123 def456 refs/heads/{branchname}"
    assert not os.path.exists(envdir)
    deploy_environments.process_line(
        line,
        basedir,
        cachedir,
        g10k_config_file=configfile,
    )

    assert os.path.isdir(envdir)

    # export the Git tree so we can compare it with the created environment
    archivedir = tempfile.mkdtemp(dir=tmp_path)
    subprocess.run(
        f"git -C {repodir} archive --format=tar {branchname}" f"| tar -C {archivedir} -xf -",
        shell=True,
        check=True,
    )

    # compare the Git extracted archive and the created environment
    comparison = filecmp.dircmp(archivedir, envdir)
    identical = (
        not comparison.diff_files
        and not comparison.left_only
        and set(comparison.right_only)
        == set(
            [
                ".g10k-deploy.json",
                "modules",
            ]
        )
    )

    assert identical is True


def test_process_line_generates_types(branchname, basedir, cachedir, envdir, configfile, monkeypatch):
    """
    Updating a branch creates the .resource_types directory in the environment.
    """
    line = f"abc123 def456 refs/heads/{branchname}"
    assert not os.path.exists(envdir)
    monkeypatch.setattr(deploy_environments, "PUPPET_CODEDIR", os.path.dirname(os.path.dirname(envdir)))
    deploy_environments.process_line(
        line,
        basedir,
        cachedir,
        g10k_config_file=configfile,
    )

    assert os.path.isdir(os.path.join(envdir, ".resource_types"))


def test_process_line_deletes_environment(branchname, basedir, cachedir, envdir, configfile):
    "Deleting a branch deletes the environment."
    os.makedirs(envdir)
    line = f"abc123 000000 refs/heads/{branchname}"
    deploy_environments.process_line(
        line,
        basedir,
        cachedir,
        g10k_config_file=configfile,
    )
    assert not os.path.exists(envdir)
