# @summary
#   Send Tails website news to a mailing list
#   This needs to be refactored to a separate module/profile
#
# @param public_hostname
#   the site's hostname
#
# @param user
#   the posix user who should be running rss2email
#
# @param home_dir
#   said user's homedir
#
# @param email_recipient
#   to whom to mail the rss feed
#
# @param email_sender
#   as whom to send mail
#
# @param feed_name
#   the name of the rss feed
#
# @param feed_path
#   the path of the rss feed
#
class profile::tails::website::rss2email (
  Stdlib::Fqdn $public_hostname,
  String $user,
  Stdlib::Absolutepath $home_dir,
  String $email_recipient,
  String $email_sender = 'info@tails.net',
  String $feed_name    = 'amnesia-news',
  String $feed_path    = 'news/emails.en.rss',
) {
  assert_private()

  include profile::tails::sysadmins

  if versioncmp($facts['os']['distro']['release']['major'], '11') < 0 {
    fail('The profile::tails::website::rss2email class only supports Debian Bullseye and newer.')
  }

  $bullseye_only_ensure = $facts['os']['distro']['release']['major'] ? {
    '11'    => present,
    default => absent,
  }

  $feed_url = "https://${public_hostname}/${feed_path}"
  $dirs = [
    "${home_dir}/.config",
    "${home_dir}/.local",
    "${home_dir}/.local/share",
  ]
  $config_file = "${home_dir}/.config/rss2email.cfg"
  $database = "${home_dir}/.local/share/rss2email.json"

  # Ensure we have rss2email 3.13 or newer, which can send multipart/alternative
  # i.e. HTML + plain text email (tails/tails#16723)
  apt::pin { 'rss2email':
    ensure     => $bullseye_only_ensure,
    packages   => ['rss2email', 'python3-feedparser'],
    originator => 'Debian',
    codename   => 'bookworm',
    priority   => 991,
  }

  package { 'rss2email':
    ensure  => present,
    require => Apt::Pin['rss2email'],
  }

  file { $dirs:
    ensure => directory,
    owner  => $user,
    group  => $user,
    mode   => '0700',
  }

  exec { "init r2e for ${public_hostname}":
    command => "/usr/bin/r2e add '${feed_name}' '${feed_url}' '${email_recipient}'",
    user    => $user,
    require => [
      Package['rss2email'],
      File[$dirs],
    ],
    creates => $database,
  }

  $settings = {
    'url'            => $feed_url,
    'from'           => $email_sender,
    'to'             => $email_recipient,
    'name-format'    => '{feed-title}',
    'html-mail'      => 'True',
    'multipart-html' => 'True',
  }

  inifile::create_ini_settings(
    { "feed.${feed_name}" => $settings },
    {
      path    => $config_file,
      require => Exec["init r2e for ${public_hostname}"],
    }
  )

  cron { "r2e run for ${public_hostname}":
    command     => '/usr/bin/chronic /usr/bin/r2e run',
    environment => "MAILTO=${profile::tails::sysadmins::email}",
    user        => $user,
    hour        => '*',
    minute      => '*/10',
    require     => keys($settings).map |$setting| {
      Ini_setting["${config_file} [feed.${feed_name}] ${setting}"]
    },
  }
}
